#include <iostream>
#include <fstream>
#include <string>
#include <streambuf>
#include "cryptopp/base64.h"
#include "cryptopp/sha.h"
#include "cryptopp/rsa.h"
#include "cryptopp/files.h"
using namespace CryptoPP;

int main(int, char**)
{
    // Get the message to verify.
    std::ifstream infile("signed.rsconfig");
    std::string message((std::istreambuf_iterator<char>(infile)), 
                         std::istreambuf_iterator<char>());
    std::cout << "Message to verify:\n----------" << std::endl << message << std::endl << "----------" << std::endl;

    // Get the signature.
    std::string signature = message;
    std::size_t found = signature.find("signature=\"");
    signature = signature.substr(found + 11);
    found = signature.find("\"");
    signature = signature.substr(0, found);
    std::cout << "\nSignature:\n----------" << std::endl << signature << std::endl << "----------" << std::endl;

    // Simplify the file.
    std::string toVerify = message;
    toVerify = toVerify.replace(toVerify.find(signature), signature.length(), "[SIGNATURE]");
    toVerify.erase(std::remove_if(toVerify.begin(), toVerify.end(), ::isspace), toVerify.end());
    std::cout << "\nData to verify:\n----------" << std::endl << toVerify << std::endl << "----------" << std::endl;

    // Verify the signature.
    std::string decodedSignature;
    std::replace(signature.begin(), signature.end(), '-', '+');
    std::replace(signature.begin(), signature.end(), '_', '/');
    Base64Decoder decoder;
    decoder.Put((byte*)signature.data(), signature.size());
    decoder.MessageEnd();
    word64 size = decoder.MaxRetrievable();
    decodedSignature.resize(size);
    decoder.Get((byte*)decodedSignature.data(), decodedSignature.size());
    
    RSA::PublicKey pubkey;
    pubkey.Load(
        FileSource("footscan9_license_public_key.der", true, NULL, true).Ref()
    );

    RSASS<PKCS1v15, SHA512>::Verifier verifier(pubkey);
    bool result = verifier.VerifyMessage((const byte *)toVerify.c_str(), toVerify.length(), (const byte *)decodedSignature.c_str(), decodedSignature.length());

    if(result)
    {
        std::cout << "The message was correctly signed." << std::endl;
    }
    else
    {
        std::cout << "The message was not correctly signed." << std::endl;
    }
}
